<?php

namespace Vidgyor\Schedule\Policies;

use Litepie\User\Contracts\UserPolicy;
use Vidgyor\Schedule\Models\Schedule;

class SchedulePolicy
{

    /**
     * Determine if the given user can view the schedule.
     *
     * @param UserPolicy $user
     * @param Schedule $schedule
     *
     * @return bool
     */
    public function view(UserPolicy $user, Schedule $schedule)
    {
        if ($user->canDo('schedule.schedule.view') && $user->isAdmin()) {
            return true;
        }

        return $schedule->user_id == user_id() && $schedule->user_type == user_type();
    }

    /**
     * Determine if the given user can create a schedule.
     *
     * @param UserPolicy $user
     * @param Schedule $schedule
     *
     * @return bool
     */
    public function create(UserPolicy $user)
    {
        return  $user->canDo('schedule.schedule.create');
    }

    /**
     * Determine if the given user can update the given schedule.
     *
     * @param UserPolicy $user
     * @param Schedule $schedule
     *
     * @return bool
     */
    public function update(UserPolicy $user, Schedule $schedule)
    {
        if ($user->canDo('schedule.schedule.edit') && $user->isAdmin()) {
            return true;
        }

        return $schedule->user_id == user_id() && $schedule->user_type == user_type();
    }

    /**
     * Determine if the given user can delete the given schedule.
     *
     * @param UserPolicy $user
     * @param Schedule $schedule
     *
     * @return bool
     */
    public function destroy(UserPolicy $user, Schedule $schedule)
    {
        return $schedule->user_id == user_id() && $schedule->user_type == user_type();
    }

    /**
     * Determine if the given user can verify the given schedule.
     *
     * @param UserPolicy $user
     * @param Schedule $schedule
     *
     * @return bool
     */
    public function verify(UserPolicy $user, Schedule $schedule)
    {
        if ($user->canDo('schedule.schedule.verify')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given schedule.
     *
     * @param UserPolicy $user
     * @param Schedule $schedule
     *
     * @return bool
     */
    public function approve(UserPolicy $user, Schedule $schedule)
    {
        if ($user->canDo('schedule.schedule.approve')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
